package com.sun.security.sasl.ntlm;

import javax.security.sasl.SaslServer;
import javax.security.sasl.SaslException;
import com.sun.security.sasl.util.PolicyUtils;
import javax.security.sasl.SaslClient;
import javax.security.auth.callback.CallbackHandler;
import java.util.Map;
import javax.security.sasl.SaslServerFactory;
import javax.security.sasl.SaslClientFactory;

public final class FactoryImpl implements SaslClientFactory, SaslServerFactory
{
    private static final String[] myMechs;
    private static final int[] mechPolicies;
    
    public FactoryImpl() {
    }
    
    @Override
    public SaslClient createSaslClient(String[] array, String s, String s2, String s3, Map<String, ?> props, CallbackHandler callbackHandler) throws SaslException {
        int i = 0;
        while (i < array.length) {
            if (array[i].equals("NTLM") && PolicyUtils.checkPolicy(FactoryImpl.mechPolicies[0], props)) {
                if (callbackHandler == null) {
                    throw new SaslException("Callback handler with support for RealmCallback, NameCallback, and PasswordCallback required");
                }
                return new NTLMClient(array[i], s, s2, s3, props, callbackHandler);
            }
            else {
                ++i;
            }
        }
        return null;
    }
    
    @Override
    public SaslServer createSaslServer(String s, String s2, String s3, Map<String, ?> props, CallbackHandler callbackHandler) throws SaslException {
        if (s.equals("NTLM") && PolicyUtils.checkPolicy(FactoryImpl.mechPolicies[0], props)) {
            if (props != null) {
                String s4 = (String)props.get("javax.security.sasl.qop");
                if (s4 != null && !s4.equals("auth")) {
                    throw new SaslException("NTLM only support auth");
                }
            }
            if (callbackHandler == null) {
                throw new SaslException("Callback handler with support for RealmCallback, NameCallback, and PasswordCallback required");
            }
            return new NTLMServer(s, s2, s3, props, callbackHandler);
        }
        else {
            return null;
        }
    }
    
    @Override
    public String[] getMechanismNames(Map<String, ?> props) {
        return PolicyUtils.filterMechs(FactoryImpl.myMechs, FactoryImpl.mechPolicies, props);
    }
    
    static {
        myMechs = new String[] { "NTLM" };
        mechPolicies = new int[] { 17 };
    }
}
